==> mi4p_1.1.tar.gz: mi4p/R/hid_ebayes.R <==
#' @title MI-aware Modifed eBayes Function
#' 
#' @description Modified eBayes function to be used instead of the one, \code{.ebayes}, implemented in the limma package
#'
#' @param fit an MArrayLM fitted model object produced by lmFit or contrasts.fit. For ebayes only, fit can alternatively be an unclassed list produced by lm.series, gls.series or mrlm containing components coefficients, stdev.unscaled, sigma and df.residual.
#' @param VarRubin a variance-covariance matrix.
#' @param mod TRUE (not used at the moment)
#' @param proportion numeric value between 0 and 1, assumed proportion of genes which are differentially expressed
#' @param stdev.coef.lim numeric vector of length 2, assumed lower and upper limits for the standard deviation of log2-fold-changes for differentially expressed genes
#' @param trend logical, should an intensity-trend be allowed for the prior variance? Default is that the prior variance is constant.
#' @param robust logical, should the estimation of df.prior and var.prior be robustified against outlier sample variances?
#' @param winsor.tail.p numeric vector of length 1 or 2, giving left and right tail proportions of x to Winsorize. Used only when robust=TRUE.
#'
#' @return eBayes produces an object of class MArrayLM (see MArrayLM-class) containing everything found in \code{fit} plus the following added components:
#' \describe{ 
#' \item{t}{numeric matrix of moderated t-statistics.}
#' \item{p.value}{numeric matrix of two-sided p-values corresponding to the t-statistics.}
#' \item{lods}{numeric matrix giving the log-odds of differential expression (on the natural log scale).}
#' \item{s2.prior}{estimated prior value for sigma^2. A row-wise vector if covariate is non-NULL, otherwise a single value.}
#' \item{df.prior}{degrees of freedom associated with s2.prior. A row-wise vector if robust=TRUE, otherwise a single value.}
#' \item{df.total}{row-wise numeric vector giving the total degrees of freedom associated with the t-statistics for each gene. Equal to df.prior+df.residual or sum(df.residual), whichever is smaller.}
#' \item{s2.post}{row-wise numeric vector giving the posterior values for sigma^2.}
#' \item{var.prior}{column-wise numeric vector giving estimated prior values for the variance of the log2-fold-changes for differentially expressed gene for each constrast. Used for evaluating lods.}
#' \item{F}{row-wise numeric vector of moderated F-statistics for testing all contrasts defined by the columns of fit simultaneously equal to zero.}
#' \item{F.p.value}{row-wise numeric vector giving p-values corresponding to F.}
#' }
#'
#'The matrices t, p.value and lods have the same dimensions as the input object fit, with rows corresponding to genes and columns to coefficients or contrasts. The vectors s2.prior, df.prior, df.total, F and F.p.value correspond to rows, with length equal to the number of genes. The vector var.prior corresponds to columns, with length equal to the number of contrasts. If s2.prior or df.prior have length 1, then the same value applies to all genes.
#'
#'s2.prior, df.prior and var.prior contain empirical Bayes hyperparameters used to obtain df.total, s2.post and lods.
#' 
#' @author Modified by M. Chion and F. Bertrand. Original by Gordon Smyth and Davis McCarthy
#' @export
#'
#' @examples
#' library(mi4p)
#' data(datasim)
#' datasim_imp <- multi.impute(data = datasim[,-1], conditions = 
#' attr(datasim,"metadata")$Condition, method = "MLE")
#' VarRubin.matrix <- rubin2.all(datasim_imp[1:5,,],
#' attr(datasim,"metadata")$Condition)
#' set.seed(2016)
#' sigma2 <- 0.05 / rchisq(100, df=10) * 10
#' y <- datasim_imp[,,1]
#' design <- cbind(Intercept=1,Group=as.numeric(
#' attr(datasim,"metadata")$Condition)-1)
#' fit.model <- limma::lmFit(y,design)
#' hid.ebayes(fit=fit.model,VarRubin.matrix[[1]])
hid.ebayes<-function (fit, VarRubin, mod=TRUE, proportion = 0.01, stdev.coef.lim = c(0.1, 4), 
          trend = FALSE, robust = FALSE, winsor.tail.p = c(0.05, 0.1)) 
{
  coefficients <- fit$coefficients
  stdev.unscaled <- fit$stdev.unscaled
  sigma <- VarRubin #fit$sigma
  df.residual <- fit$df.residual
  if (is.null(coefficients) || is.null(stdev.unscaled) || is.null(sigma) || 
      is.null(df.residual)) 
    stop("No data, or argument is not a valid lmFit object")
  if (all(df.residual == 0)) 
    stop("No residual degrees of freedom in linear model fits")
  if (all(!is.finite(sigma))) 
    stop("No finite residual standard deviations")
  if (trend) {
    covariate <- fit$Amean
    if (is.null(covariate)) 
      stop("Need Amean component in fit to estimate trend")
  }
  else {
    covariate <- NULL
  }
  if (mod) {
    out <- limma::squeezeVar(sigma^2, df.residual, covariate = covariate, 
                    robust = robust, winsor.tail.p = winsor.tail.p)
    out$s2.prior <- out$var.prior
    out$s2.post <- out$var.post
    out$var.prior <- out$var.post <- NULL
    out$t <- coefficients/stdev.unscaled/sqrt(out$s2.post)
    df.total <- df.residual + out$df.prior
    df.pooled <- sum(df.residual, na.rm = TRUE)
    df.total <- pmin(df.total, df.pooled)
    out$df.total <- df.total
    out$p.value <- 2 * pt(-abs(out$t), df = df.total)
    var.prior.lim <- stdev.coef.lim^2/median(out$s2.prior)
    out$var.prior <- limma::tmixture.matrix(out$t, stdev.unscaled, df.total, 
                                   proportion, var.prior.lim)
  }
  else {
    #TODO
  }
  
  if (any(is.na(out$var.prior))) {
    out$var.prior[is.na(out$var.prior)] <- 1/out$s2.prior
    warning("Estimation of var.prior failed - set to default value")
  }
  r <- rep(1, NROW(out$t)) %o% out$var.prior
  r <- (stdev.unscaled^2 + r)/stdev.unscaled^2
  t2 <- out$t^2
  Infdf <- out$df.prior > 10^6
  if (any(Infdf)) {
    kernel <- t2 * (1 - 1/r)/2
    if (any(!Infdf)) {
      t2.f <- t2[!Infdf]
      r.f <- r[!Infdf]
      df.total.f <- df.total[!Infdf]
      kernel[!Infdf] <- (1 + df.total.f)/2 * log((t2.f + 
                                                    df.total.f)/(t2.f/r.f + df.total.f))
    }
  }
  else kernel <- (1 + df.total)/2 * log((t2 + df.total)/(t2/r + 
                                                           df.total))
  out$lods <- log(proportion/(1 - proportion)) - log(r)/2 + 
    kernel
  out
}

==> mi4p_1.3.tar.gz: mi4p/R/hid_ebayes.R <==
#' @title MI-aware Modifed eBayes Function
#' 
#' @description Modified eBayes function to be used instead of the one, \code{.ebayes}, implemented in the limma package
#'
#' @param fit an MArrayLM fitted model object produced by lmFit or contrasts.fit. For ebayes only, fit can alternatively be an unclassed list produced by lm.series, gls.series or mrlm containing components coefficients, stdev.unscaled, sigma and df.residual.
#' @param VarRubin a variance-covariance matrix.
#' @param mod TRUE (not used at the moment)
#' @param proportion numeric value between 0 and 1, assumed proportion of genes which are differentially expressed
#' @param stdev.coef.lim numeric vector of length 2, assumed lower and upper limits for the standard deviation of log2-fold-changes for differentially expressed genes
#' @param trend logical, should an intensity-trend be allowed for the prior variance? Default is that the prior variance is constant.
#' @param robust logical, should the estimation of df.prior and var.prior be robustified against outlier sample variances?
#' @param winsor.tail.p numeric vector of length 1 or 2, giving left and right tail proportions of x to Winsorize. Used only when robust=TRUE.
#' @param legacy boolean to stick to former way to squeeze variances. Defaults to TRUE.
#'
#' @return eBayes produces an object of class MArrayLM (see MArrayLM-class) containing everything found in \code{fit} plus the following added components:
#' \describe{ 
#' \item{t}{numeric matrix of moderated t-statistics.}
#' \item{p.value}{numeric matrix of two-sided p-values corresponding to the t-statistics.}
#' \item{lods}{numeric matrix giving the log-odds of differential expression (on the natural log scale).}
#' \item{s2.prior}{estimated prior value for sigma^2. A row-wise vector if covariate is non-NULL, otherwise a single value.}
#' \item{df.prior}{degrees of freedom associated with s2.prior. A row-wise vector if robust=TRUE, otherwise a single value.}
#' \item{df.total}{row-wise numeric vector giving the total degrees of freedom associated with the t-statistics for each gene. Equal to df.prior+df.residual or sum(df.residual), whichever is smaller.}
#' \item{s2.post}{row-wise numeric vector giving the posterior values for sigma^2.}
#' \item{var.prior}{column-wise numeric vector giving estimated prior values for the variance of the log2-fold-changes for differentially expressed gene for each constrast. Used for evaluating lods.}
#' \item{F}{row-wise numeric vector of moderated F-statistics for testing all contrasts defined by the columns of fit simultaneously equal to zero.}
#' \item{F.p.value}{row-wise numeric vector giving p-values corresponding to F.}
#' }
#'
#'The matrices t, p.value and lods have the same dimensions as the input object fit, with rows corresponding to genes and columns to coefficients or contrasts. The vectors s2.prior, df.prior, df.total, F and F.p.value correspond to rows, with length equal to the number of genes. The vector var.prior corresponds to columns, with length equal to the number of contrasts. If s2.prior or df.prior have length 1, then the same value applies to all genes.
#'
#'s2.prior, df.prior and var.prior contain empirical Bayes hyperparameters used to obtain df.total, s2.post and lods.
#' 
#' @author Modified by M. Chion and F. Bertrand. Original by Gordon Smyth and Davis McCarthy
#' @export
#'
#' @examples
#' library(mi4p)
#' data(datasim)
#' datasim_imp <- multi.impute(data = datasim[,-1], conditions = 
#' attr(datasim,"metadata")$Condition, method = "MLE")
#' VarRubin.matrix <- rubin2.all(datasim_imp[1:5,,],
#' attr(datasim,"metadata")$Condition)
#' set.seed(2016)
#' sigma2 <- 0.05 / rchisq(100, df=10) * 10
#' y <- datasim_imp[,,1]
#' design <- cbind(Intercept=1,Group=as.numeric(
#' attr(datasim,"metadata")$Condition)-1)
#' fit.model <- limma::lmFit(y,design)
#' hid.ebayes(fit=fit.model,VarRubin.matrix[[1]])
hid.ebayes<-function (fit, VarRubin, mod=TRUE, proportion = 0.01, stdev.coef.lim = c(0.1, 4), 
          trend = FALSE, robust = FALSE, winsor.tail.p = c(0.05, 0.1), legacy=TRUE) 
{
  coefficients <- fit$coefficients
  stdev.unscaled <- fit$stdev.unscaled
  sigma <- VarRubin #fit$sigma
  df.residual <- fit$df.residual
  if (is.null(coefficients) || is.null(stdev.unscaled) || is.null(sigma) || 
      is.null(df.residual)) 
    stop("No data, or argument is not a valid lmFit object")
  if (all(df.residual == 0)) 
    stop("No residual degrees of freedom in linear model fits")
  if (all(!is.finite(sigma))) 
    stop("No finite residual standard deviations")
  if (trend) {
    covariate <- fit$Amean
    if (is.null(covariate)) 
      stop("Need Amean component in fit to estimate trend")
  }
  else {
    covariate <- NULL
  }
  if (mod) {
    if(packageVersion("limma")>= "3.61.8"){
    out <- limma::squeezeVar(sigma^2, df.residual, covariate = covariate, 
                    robust = robust, winsor.tail.p = winsor.tail.p, 
                    legacy = legacy)
    } else {
      out <- limma::squeezeVar(sigma^2, df.residual, covariate = covariate, 
                               robust = robust, winsor.tail.p = winsor.tail.p)
    }
    out$s2.prior <- out$var.prior
    out$s2.post <- out$var.post
    out$var.prior <- out$var.post <- NULL
    out$t <- coefficients/stdev.unscaled/sqrt(out$s2.post)
    df.total <- df.residual + out$df.prior
    df.pooled <- sum(df.residual, na.rm = TRUE)
    df.total <- pmin(df.total, df.pooled)
    out$df.total <- df.total
    out$p.value <- 2 * pt(-abs(out$t), df = df.total)
    var.prior.lim <- stdev.coef.lim^2/median(out$s2.prior)
    out$var.prior <- limma::tmixture.matrix(out$t, stdev.unscaled, df.total, 
                                   proportion, var.prior.lim)
  }
  else {
    #TODO
  }
  
  if (any(is.na(out$var.prior))) {
    out$var.prior[is.na(out$var.prior)] <- 1/out$s2.prior
    warning("Estimation of var.prior failed - set to default value")
  }
  r <- rep(1, NROW(out$t)) %o% out$var.prior
  r <- (stdev.unscaled^2 + r)/stdev.unscaled^2
  t2 <- out$t^2
  Infdf <- out$df.prior > 10^6
  if (any(Infdf)) {
    kernel <- t2 * (1 - 1/r)/2
    if (any(!Infdf)) {
      t2.f <- t2[!Infdf]
      r.f <- r[!Infdf]
      df.total.f <- df.total[!Infdf]
      kernel[!Infdf] <- (1 + df.total.f)/2 * log((t2.f + 
                                                    df.total.f)/(t2.f/r.f + df.total.f))
    }
  }
  else kernel <- (1 + df.total)/2 * log((t2 + df.total)/(t2/r + 
                                                           df.total))
  out$lods <- log(proportion/(1 - proportion)) - log(r)/2 + 
    kernel
  out
}
